'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var sh = require('shorthash');
var sh__default = _interopDefault(sh);
var helperPluginUtils = require('@babel/helper-plugin-utils');
var core = require('@babel/core');
var codeFrame = require('@babel/code-frame');
var glob = _interopDefault(require('glob'));
var p = require('path');
var mkdirp = require('mkdirp');
var fs = require('fs');
var util = require('util');
var Excel = _interopDefault(require('exceljs'));
var child_process = require('child_process');

var genId = function genId(message, scope) {
  return sh.unique(message + (scope || ''));
};

var cloneJSXOpeningElement = function cloneJSXOpeningElement(path, descriptor, msgAttr, removeDefaultMessage) {
  return core.types.jsxOpeningElement(core.types.jsxIdentifier(path.node.name.name), [].concat(removeDefaultMessage ? path.node.attributes.filter(function (attr) {
    return attr !== msgAttr;
  }) : path.node.attributes, [core.types.jSXAttribute(core.types.jsxIdentifier('id'), core.types.stringLiteral(descriptor.id))]));
};
var cloneNormalCallExpression = function cloneNormalCallExpression(path, name, descriptor, msgProp, removeDefaultMessage) {
  var node = path.node;
  var args = node.arguments[0];
  var pathArgs = [core.types.objectExpression([].concat(removeDefaultMessage ? args.properties.filter(function (p) {
    return p !== msgProp;
  }) : args.properties, [core.types.objectProperty(core.types.identifier('id'), core.types.stringLiteral(descriptor.id))]))].concat(path.node.arguments.slice(1));

  if (core.types.isIdentifier(node.callee)) {
    return core.types.callExpression(core.types.identifier(name), pathArgs);
  }

  var memberCallee = node.callee;
  var object = memberCallee.object;
  return core.types.callExpression(core.types.memberExpression(core.types.identifier(object.name), core.types.identifier(memberCallee.property.name)), pathArgs);
};

var scopeRegexp = /@message\.scope(?:[\t\x20])+([\w-]+)/;

var praseCommments = function praseCommments(leadingComments) {
  var _leadingComments$map$;

  if (!leadingComments) {
    return null;
  }

  var scope = (_leadingComments$map$ = leadingComments.map(function (item) {
    return item.value;
  }).map(function (item) {
    var matched = item.match(scopeRegexp);
    return matched == null ? void 0 : matched[1];
  }).filter(Boolean)) == null ? void 0 : _leadingComments$map$[0];
  var matchedList = [];
  leadingComments.forEach(function (_ref) {
    var value = _ref.value;
    var data = value.split(/\n/);
    data.forEach(function (item) {
      var tmp;
      var messagesRegexp = /@message\.([a-z]{2}-[A-Z]{2})(?:[\t\x20])+([^\s@]+(?:\s+?[^\s@]+)*)/g;

      while (tmp = messagesRegexp.exec(item)) {
        matchedList.push(tmp);
      }
    });
  });
  var data = matchedList.reduce(function (result, item) {
    var locale = item[1],
        message = item[2];

    if (!locale || !message) {
      return result;
    }

    result[locale] = message;
    return result;
  }, {});
  return {
    scope: scope,
    messages: Object.keys(data).length ? data : undefined
  };
};

var ShortcutFunctionName = 'formatMessage';
var CalleeFunctionNames = ['formatMessage'];
var ComponentNames = ['FormattedMessage'];
var ScanConfigFilename = 'mes-babel-plugin-intl.config.js';
var UmiPluginKey = 'mes-babel-plugin-intl';
var I18nMessageFilename = './i18n-messages/data.json';
var LocaleMap = {
  zh: 'zh-CN',
  en: 'en-US'
};
var DistributionName = 'Distribution';
var ExcelFilename = 'translated.xlsx';
var InternalTranslatedDataFilename = 'internal-translated.js';

function createJSXMessageDescriptor(path, writeMode, removeDefaultMessage) {
  if (writeMode === void 0) {
    writeMode = true;
  }

  if (removeDefaultMessage === void 0) {
    removeDefaultMessage = false;
  }

  var attributes = path.node.attributes;
  var descriptor = {};
  var msgAttr = null;
  attributes.forEach(function (attr) {
    if (core.types.isJSXAttribute(attr) && core.types.isJSXIdentifier(attr.name) && core.types.isStringLiteral(attr.value)) {
      var _attr$value$value;

      var value = (_attr$value$value = attr.value.value) == null ? void 0 : _attr$value$value.trim();
      var name = attr.name.name;

      if (name === 'id') {
        descriptor.id = value;
      } else if (name === 'defaultMessage') {
        msgAttr = attr;
        descriptor.defaultMessage = value;
      }
    }
  });

  if (descriptor.defaultMessage) {
    if (!descriptor.id) {
      descriptor.id = genId(descriptor.defaultMessage);

      if (writeMode) {
        // 原地加上 id
        path.replaceWith(cloneJSXOpeningElement(path, descriptor, msgAttr, removeDefaultMessage));
        path.skip();
      }
    } else if (removeDefaultMessage) {
      path.replaceWith(cloneJSXOpeningElement(path, descriptor, msgAttr, true));
      path.skip();
    }

    return descriptor;
  }

  return null;
}

function createFunctionMessageDescriptor(path, shortcut, writeMode, removeDefaultMessage) {
  if (writeMode === void 0) {
    writeMode = true;
  }

  if (removeDefaultMessage === void 0) {
    removeDefaultMessage = false;
  }

  var node = path.node;
  var name = core.types.isIdentifier(node.callee) ? node.callee.name : node.callee.property.name;
  var args = node.arguments[0];
  var restArgs = node.arguments[1];
  var maybeIdArgs = node.arguments[2];
  var descriptor = {};
  var msgProp = null;
  var hasQuery = false;
  var translatedMessage = praseCommments(node.leadingComments);

  if (core.types.isIdentifier(node.callee) && typeof shortcut === 'string' && name === shortcut && core.types.isStringLiteral(args) && args.value.trim()) {
    descriptor.defaultMessage = args.value.trim();

    if (core.types.isStringLiteral(restArgs) || core.types.isObjectExpression(restArgs) && restArgs.properties.length > 0) {
      hasQuery = core.types.isObjectExpression(restArgs) && restArgs.properties.length > 0;
      descriptor.id = core.types.isStringLiteral(restArgs) ? restArgs.value.trim() : core.types.isStringLiteral(maybeIdArgs) ? maybeIdArgs.value.trim() : '';
    }

    if (translatedMessage != null && translatedMessage.messages) {
      descriptor.translatedMessage = translatedMessage.messages;
    }

    if (!descriptor.id) {
      descriptor.id = genId(descriptor.defaultMessage, translatedMessage == null ? void 0 : translatedMessage.scope);

      if (writeMode) {
        path.replaceWith(core.types.callExpression(core.types.identifier(name), [args].concat(hasQuery ? [restArgs, core.types.stringLiteral(descriptor.id)] : [core.types.stringLiteral(descriptor.id)])));
        path.skip();
      }
    }

    return descriptor;
  }

  if (!core.types.isObjectExpression(args)) {
    return null;
  }

  args.properties.forEach(function (prop) {
    if (!core.types.isObjectProperty(prop)) {
      return;
    }

    var key = prop.key,
        value = prop.value;

    if (core.types.isIdentifier(key) && core.types.isStringLiteral(value)) {
      var val = value.value.trim();

      if (key.name === 'id') {
        descriptor.id = val;
      } else if (key.name === 'defaultMessage') {
        descriptor.defaultMessage = val;
        msgProp = prop;
      }
    }
  });

  if (translatedMessage != null && translatedMessage.messages) {
    descriptor.translatedMessage = translatedMessage == null ? void 0 : translatedMessage.messages;
  }

  if (descriptor.defaultMessage) {
    if (!descriptor.id) {
      descriptor.id = genId(descriptor.defaultMessage, translatedMessage == null ? void 0 : translatedMessage.scope); // insert id

      if (writeMode) {
        path.replaceWith(cloneNormalCallExpression(path, name, descriptor, msgProp, removeDefaultMessage));
        path.skip();
      }
    } else if (removeDefaultMessage) {
      path.replaceWith(cloneNormalCallExpression(path, name, descriptor, msgProp, true));
      path.skip();
    }

    return descriptor;
  }

  return null;
}

function checkRepeatedDescriptor(path, descriptor, storage) {
  var prev = storage.get(descriptor.id);

  if (prev != null && prev.length) {
    prev.forEach(function (item) {
      if (item.defaultMessage !== descriptor.defaultMessage) {
        var result = codeFrame.codeFrameColumns(path.hub.getCode(), path.node.loc);
        console.log('There are duplicate id, different messages:');
        console.log("ID:             " + item.id);
        console.log("defaultMessage: " + item.defaultMessage);
        console.log(result);
      }
    });
  }
}

function run(writeMode, options, storage) {
  var componentNames = ComponentNames.concat(options.componentNames || []);
  var functionNames = CalleeFunctionNames.concat(options.functionNames || []);
  var shortcut = 'shortcut' in options ? typeof options.shortcut === 'boolean' && options.shortcut ? ShortcutFunctionName : typeof options.shortcut === 'object' ? options.shortcut.functionName : '' : ShortcutFunctionName;
  return {
    pre: function pre() {
      if (storage) {
        storage.set(this.filename, new Map());
      }
    },
    visitor: {
      JSXOpeningElement: function JSXOpeningElement(path, file) {
        var name = path.node.name;
        var filename = file.filename;

        if (core.types.isJSXIdentifier(name) && componentNames.includes(name.name)) {
          var descriptor = createJSXMessageDescriptor(path, writeMode, options.removeDefaultMessage);

          if (descriptor && storage) {
            if (options.fileLoc) {
              descriptor.filename = filename;
              descriptor.loc = path.node.loc;
            }

            var container = storage.get(filename);
            checkRepeatedDescriptor(path, descriptor, container);

            if (container.has(descriptor.id)) {
              var _container$get;

              (_container$get = container.get(descriptor.id)) == null ? void 0 : _container$get.push(descriptor);
            } else {
              container.set(descriptor.id, [descriptor]);
            }
          }
        }
      },
      CallExpression: function CallExpression(path, file) {
        var callee = path.node.callee;
        var filename = file.filename;

        if (core.types.isIdentifier(callee) && functionNames.includes(callee.name) || core.types.isMemberExpression(callee) && core.types.isIdentifier(callee.property) && core.types.isIdentifier(callee.object) && functionNames.includes(callee.property.name)) {
          var descriptor = createFunctionMessageDescriptor(path, shortcut, writeMode, options.removeDefaultMessage);

          if (descriptor && storage) {
            if (options.fileLoc) {
              descriptor.filename = filename;
              descriptor.loc = path.node.loc;
            }

            var container = storage.get(filename);
            checkRepeatedDescriptor(path, descriptor, container);

            if (container.has(descriptor.id)) {
              var _container$get2;

              (_container$get2 = container.get(descriptor.id)) == null ? void 0 : _container$get2.push(descriptor);
            } else {
              container.set(descriptor.id, [descriptor]);
            }
          }
        }
      }
    }
  };
}

var plugin = /*#__PURE__*/helperPluginUtils.declare(function (_, options) {
  return run(true, options);
});

var umiPlugin = (function (api) {
  api.describe({
    key: UmiPluginKey,
    config: {
      "default": undefined,
      schema: function schema(joi) {
        return joi.object({
          functionNames: joi.array().items(joi.string()),
          componentNames: joi.array().items(joi.string()),
          fileLoc: joi["boolean"](),
          removeDefaultMessage: joi["boolean"](),
          shortcut: [joi["boolean"](), joi.object({
            functionName: joi.string()
          })]
        });
      }
    }
  });
  api.modifyBabelOpts(function (babelOpts) {
    var _api$config;

    babelOpts.plugins.push([plugin, api == null ? void 0 : (_api$config = api.config) == null ? void 0 : _api$config[UmiPluginKey]]);
    return babelOpts;
  });
});

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
  try {
    var info = gen[key](arg);
    var value = info.value;
  } catch (error) {
    reject(error);
    return;
  }

  if (info.done) {
    resolve(value);
  } else {
    Promise.resolve(value).then(_next, _throw);
  }
}

function _asyncToGenerator(fn) {
  return function () {
    var self = this,
        args = arguments;
    return new Promise(function (resolve, reject) {
      var gen = fn.apply(self, args);

      function _next(value) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
      }

      function _throw(err) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
      }

      _next(undefined);
    });
  };
}

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _createForOfIteratorHelperLoose(o, allowArrayLike) {
  var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"];
  if (it) return (it = it.call(o)).next.bind(it);

  if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
    if (it) o = it;
    var i = 0;
    return function () {
      if (i >= o.length) return {
        done: true
      };
      return {
        done: false,
        value: o[i++]
      };
    };
  }

  throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var runtime_1 = createCommonjsModule(function (module) {
/**
 * Copyright (c) 2014-present, Facebook, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

var runtime = (function (exports) {

  var Op = Object.prototype;
  var hasOwn = Op.hasOwnProperty;
  var undefined$1; // More compressible than void 0.
  var $Symbol = typeof Symbol === "function" ? Symbol : {};
  var iteratorSymbol = $Symbol.iterator || "@@iterator";
  var asyncIteratorSymbol = $Symbol.asyncIterator || "@@asyncIterator";
  var toStringTagSymbol = $Symbol.toStringTag || "@@toStringTag";

  function define(obj, key, value) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
    return obj[key];
  }
  try {
    // IE 8 has a broken Object.defineProperty that only works on DOM objects.
    define({}, "");
  } catch (err) {
    define = function(obj, key, value) {
      return obj[key] = value;
    };
  }

  function wrap(innerFn, outerFn, self, tryLocsList) {
    // If outerFn provided and outerFn.prototype is a Generator, then outerFn.prototype instanceof Generator.
    var protoGenerator = outerFn && outerFn.prototype instanceof Generator ? outerFn : Generator;
    var generator = Object.create(protoGenerator.prototype);
    var context = new Context(tryLocsList || []);

    // The ._invoke method unifies the implementations of the .next,
    // .throw, and .return methods.
    generator._invoke = makeInvokeMethod(innerFn, self, context);

    return generator;
  }
  exports.wrap = wrap;

  // Try/catch helper to minimize deoptimizations. Returns a completion
  // record like context.tryEntries[i].completion. This interface could
  // have been (and was previously) designed to take a closure to be
  // invoked without arguments, but in all the cases we care about we
  // already have an existing method we want to call, so there's no need
  // to create a new function object. We can even get away with assuming
  // the method takes exactly one argument, since that happens to be true
  // in every case, so we don't have to touch the arguments object. The
  // only additional allocation required is the completion record, which
  // has a stable shape and so hopefully should be cheap to allocate.
  function tryCatch(fn, obj, arg) {
    try {
      return { type: "normal", arg: fn.call(obj, arg) };
    } catch (err) {
      return { type: "throw", arg: err };
    }
  }

  var GenStateSuspendedStart = "suspendedStart";
  var GenStateSuspendedYield = "suspendedYield";
  var GenStateExecuting = "executing";
  var GenStateCompleted = "completed";

  // Returning this object from the innerFn has the same effect as
  // breaking out of the dispatch switch statement.
  var ContinueSentinel = {};

  // Dummy constructor functions that we use as the .constructor and
  // .constructor.prototype properties for functions that return Generator
  // objects. For full spec compliance, you may wish to configure your
  // minifier not to mangle the names of these two functions.
  function Generator() {}
  function GeneratorFunction() {}
  function GeneratorFunctionPrototype() {}

  // This is a polyfill for %IteratorPrototype% for environments that
  // don't natively support it.
  var IteratorPrototype = {};
  IteratorPrototype[iteratorSymbol] = function () {
    return this;
  };

  var getProto = Object.getPrototypeOf;
  var NativeIteratorPrototype = getProto && getProto(getProto(values([])));
  if (NativeIteratorPrototype &&
      NativeIteratorPrototype !== Op &&
      hasOwn.call(NativeIteratorPrototype, iteratorSymbol)) {
    // This environment has a native %IteratorPrototype%; use it instead
    // of the polyfill.
    IteratorPrototype = NativeIteratorPrototype;
  }

  var Gp = GeneratorFunctionPrototype.prototype =
    Generator.prototype = Object.create(IteratorPrototype);
  GeneratorFunction.prototype = Gp.constructor = GeneratorFunctionPrototype;
  GeneratorFunctionPrototype.constructor = GeneratorFunction;
  GeneratorFunction.displayName = define(
    GeneratorFunctionPrototype,
    toStringTagSymbol,
    "GeneratorFunction"
  );

  // Helper for defining the .next, .throw, and .return methods of the
  // Iterator interface in terms of a single ._invoke method.
  function defineIteratorMethods(prototype) {
    ["next", "throw", "return"].forEach(function(method) {
      define(prototype, method, function(arg) {
        return this._invoke(method, arg);
      });
    });
  }

  exports.isGeneratorFunction = function(genFun) {
    var ctor = typeof genFun === "function" && genFun.constructor;
    return ctor
      ? ctor === GeneratorFunction ||
        // For the native GeneratorFunction constructor, the best we can
        // do is to check its .name property.
        (ctor.displayName || ctor.name) === "GeneratorFunction"
      : false;
  };

  exports.mark = function(genFun) {
    if (Object.setPrototypeOf) {
      Object.setPrototypeOf(genFun, GeneratorFunctionPrototype);
    } else {
      genFun.__proto__ = GeneratorFunctionPrototype;
      define(genFun, toStringTagSymbol, "GeneratorFunction");
    }
    genFun.prototype = Object.create(Gp);
    return genFun;
  };

  // Within the body of any async function, `await x` is transformed to
  // `yield regeneratorRuntime.awrap(x)`, so that the runtime can test
  // `hasOwn.call(value, "__await")` to determine if the yielded value is
  // meant to be awaited.
  exports.awrap = function(arg) {
    return { __await: arg };
  };

  function AsyncIterator(generator, PromiseImpl) {
    function invoke(method, arg, resolve, reject) {
      var record = tryCatch(generator[method], generator, arg);
      if (record.type === "throw") {
        reject(record.arg);
      } else {
        var result = record.arg;
        var value = result.value;
        if (value &&
            typeof value === "object" &&
            hasOwn.call(value, "__await")) {
          return PromiseImpl.resolve(value.__await).then(function(value) {
            invoke("next", value, resolve, reject);
          }, function(err) {
            invoke("throw", err, resolve, reject);
          });
        }

        return PromiseImpl.resolve(value).then(function(unwrapped) {
          // When a yielded Promise is resolved, its final value becomes
          // the .value of the Promise<{value,done}> result for the
          // current iteration.
          result.value = unwrapped;
          resolve(result);
        }, function(error) {
          // If a rejected Promise was yielded, throw the rejection back
          // into the async generator function so it can be handled there.
          return invoke("throw", error, resolve, reject);
        });
      }
    }

    var previousPromise;

    function enqueue(method, arg) {
      function callInvokeWithMethodAndArg() {
        return new PromiseImpl(function(resolve, reject) {
          invoke(method, arg, resolve, reject);
        });
      }

      return previousPromise =
        // If enqueue has been called before, then we want to wait until
        // all previous Promises have been resolved before calling invoke,
        // so that results are always delivered in the correct order. If
        // enqueue has not been called before, then it is important to
        // call invoke immediately, without waiting on a callback to fire,
        // so that the async generator function has the opportunity to do
        // any necessary setup in a predictable way. This predictability
        // is why the Promise constructor synchronously invokes its
        // executor callback, and why async functions synchronously
        // execute code before the first await. Since we implement simple
        // async functions in terms of async generators, it is especially
        // important to get this right, even though it requires care.
        previousPromise ? previousPromise.then(
          callInvokeWithMethodAndArg,
          // Avoid propagating failures to Promises returned by later
          // invocations of the iterator.
          callInvokeWithMethodAndArg
        ) : callInvokeWithMethodAndArg();
    }

    // Define the unified helper method that is used to implement .next,
    // .throw, and .return (see defineIteratorMethods).
    this._invoke = enqueue;
  }

  defineIteratorMethods(AsyncIterator.prototype);
  AsyncIterator.prototype[asyncIteratorSymbol] = function () {
    return this;
  };
  exports.AsyncIterator = AsyncIterator;

  // Note that simple async functions are implemented on top of
  // AsyncIterator objects; they just return a Promise for the value of
  // the final result produced by the iterator.
  exports.async = function(innerFn, outerFn, self, tryLocsList, PromiseImpl) {
    if (PromiseImpl === void 0) PromiseImpl = Promise;

    var iter = new AsyncIterator(
      wrap(innerFn, outerFn, self, tryLocsList),
      PromiseImpl
    );

    return exports.isGeneratorFunction(outerFn)
      ? iter // If outerFn is a generator, return the full iterator.
      : iter.next().then(function(result) {
          return result.done ? result.value : iter.next();
        });
  };

  function makeInvokeMethod(innerFn, self, context) {
    var state = GenStateSuspendedStart;

    return function invoke(method, arg) {
      if (state === GenStateExecuting) {
        throw new Error("Generator is already running");
      }

      if (state === GenStateCompleted) {
        if (method === "throw") {
          throw arg;
        }

        // Be forgiving, per 25.3.3.3.3 of the spec:
        // https://people.mozilla.org/~jorendorff/es6-draft.html#sec-generatorresume
        return doneResult();
      }

      context.method = method;
      context.arg = arg;

      while (true) {
        var delegate = context.delegate;
        if (delegate) {
          var delegateResult = maybeInvokeDelegate(delegate, context);
          if (delegateResult) {
            if (delegateResult === ContinueSentinel) continue;
            return delegateResult;
          }
        }

        if (context.method === "next") {
          // Setting context._sent for legacy support of Babel's
          // function.sent implementation.
          context.sent = context._sent = context.arg;

        } else if (context.method === "throw") {
          if (state === GenStateSuspendedStart) {
            state = GenStateCompleted;
            throw context.arg;
          }

          context.dispatchException(context.arg);

        } else if (context.method === "return") {
          context.abrupt("return", context.arg);
        }

        state = GenStateExecuting;

        var record = tryCatch(innerFn, self, context);
        if (record.type === "normal") {
          // If an exception is thrown from innerFn, we leave state ===
          // GenStateExecuting and loop back for another invocation.
          state = context.done
            ? GenStateCompleted
            : GenStateSuspendedYield;

          if (record.arg === ContinueSentinel) {
            continue;
          }

          return {
            value: record.arg,
            done: context.done
          };

        } else if (record.type === "throw") {
          state = GenStateCompleted;
          // Dispatch the exception by looping back around to the
          // context.dispatchException(context.arg) call above.
          context.method = "throw";
          context.arg = record.arg;
        }
      }
    };
  }

  // Call delegate.iterator[context.method](context.arg) and handle the
  // result, either by returning a { value, done } result from the
  // delegate iterator, or by modifying context.method and context.arg,
  // setting context.delegate to null, and returning the ContinueSentinel.
  function maybeInvokeDelegate(delegate, context) {
    var method = delegate.iterator[context.method];
    if (method === undefined$1) {
      // A .throw or .return when the delegate iterator has no .throw
      // method always terminates the yield* loop.
      context.delegate = null;

      if (context.method === "throw") {
        // Note: ["return"] must be used for ES3 parsing compatibility.
        if (delegate.iterator["return"]) {
          // If the delegate iterator has a return method, give it a
          // chance to clean up.
          context.method = "return";
          context.arg = undefined$1;
          maybeInvokeDelegate(delegate, context);

          if (context.method === "throw") {
            // If maybeInvokeDelegate(context) changed context.method from
            // "return" to "throw", let that override the TypeError below.
            return ContinueSentinel;
          }
        }

        context.method = "throw";
        context.arg = new TypeError(
          "The iterator does not provide a 'throw' method");
      }

      return ContinueSentinel;
    }

    var record = tryCatch(method, delegate.iterator, context.arg);

    if (record.type === "throw") {
      context.method = "throw";
      context.arg = record.arg;
      context.delegate = null;
      return ContinueSentinel;
    }

    var info = record.arg;

    if (! info) {
      context.method = "throw";
      context.arg = new TypeError("iterator result is not an object");
      context.delegate = null;
      return ContinueSentinel;
    }

    if (info.done) {
      // Assign the result of the finished delegate to the temporary
      // variable specified by delegate.resultName (see delegateYield).
      context[delegate.resultName] = info.value;

      // Resume execution at the desired location (see delegateYield).
      context.next = delegate.nextLoc;

      // If context.method was "throw" but the delegate handled the
      // exception, let the outer generator proceed normally. If
      // context.method was "next", forget context.arg since it has been
      // "consumed" by the delegate iterator. If context.method was
      // "return", allow the original .return call to continue in the
      // outer generator.
      if (context.method !== "return") {
        context.method = "next";
        context.arg = undefined$1;
      }

    } else {
      // Re-yield the result returned by the delegate method.
      return info;
    }

    // The delegate iterator is finished, so forget it and continue with
    // the outer generator.
    context.delegate = null;
    return ContinueSentinel;
  }

  // Define Generator.prototype.{next,throw,return} in terms of the
  // unified ._invoke helper method.
  defineIteratorMethods(Gp);

  define(Gp, toStringTagSymbol, "Generator");

  // A Generator should always return itself as the iterator object when the
  // @@iterator function is called on it. Some browsers' implementations of the
  // iterator prototype chain incorrectly implement this, causing the Generator
  // object to not be returned from this call. This ensures that doesn't happen.
  // See https://github.com/facebook/regenerator/issues/274 for more details.
  Gp[iteratorSymbol] = function() {
    return this;
  };

  Gp.toString = function() {
    return "[object Generator]";
  };

  function pushTryEntry(locs) {
    var entry = { tryLoc: locs[0] };

    if (1 in locs) {
      entry.catchLoc = locs[1];
    }

    if (2 in locs) {
      entry.finallyLoc = locs[2];
      entry.afterLoc = locs[3];
    }

    this.tryEntries.push(entry);
  }

  function resetTryEntry(entry) {
    var record = entry.completion || {};
    record.type = "normal";
    delete record.arg;
    entry.completion = record;
  }

  function Context(tryLocsList) {
    // The root entry object (effectively a try statement without a catch
    // or a finally block) gives us a place to store values thrown from
    // locations where there is no enclosing try statement.
    this.tryEntries = [{ tryLoc: "root" }];
    tryLocsList.forEach(pushTryEntry, this);
    this.reset(true);
  }

  exports.keys = function(object) {
    var keys = [];
    for (var key in object) {
      keys.push(key);
    }
    keys.reverse();

    // Rather than returning an object with a next method, we keep
    // things simple and return the next function itself.
    return function next() {
      while (keys.length) {
        var key = keys.pop();
        if (key in object) {
          next.value = key;
          next.done = false;
          return next;
        }
      }

      // To avoid creating an additional object, we just hang the .value
      // and .done properties off the next function object itself. This
      // also ensures that the minifier will not anonymize the function.
      next.done = true;
      return next;
    };
  };

  function values(iterable) {
    if (iterable) {
      var iteratorMethod = iterable[iteratorSymbol];
      if (iteratorMethod) {
        return iteratorMethod.call(iterable);
      }

      if (typeof iterable.next === "function") {
        return iterable;
      }

      if (!isNaN(iterable.length)) {
        var i = -1, next = function next() {
          while (++i < iterable.length) {
            if (hasOwn.call(iterable, i)) {
              next.value = iterable[i];
              next.done = false;
              return next;
            }
          }

          next.value = undefined$1;
          next.done = true;

          return next;
        };

        return next.next = next;
      }
    }

    // Return an iterator with no values.
    return { next: doneResult };
  }
  exports.values = values;

  function doneResult() {
    return { value: undefined$1, done: true };
  }

  Context.prototype = {
    constructor: Context,

    reset: function(skipTempReset) {
      this.prev = 0;
      this.next = 0;
      // Resetting context._sent for legacy support of Babel's
      // function.sent implementation.
      this.sent = this._sent = undefined$1;
      this.done = false;
      this.delegate = null;

      this.method = "next";
      this.arg = undefined$1;

      this.tryEntries.forEach(resetTryEntry);

      if (!skipTempReset) {
        for (var name in this) {
          // Not sure about the optimal order of these conditions:
          if (name.charAt(0) === "t" &&
              hasOwn.call(this, name) &&
              !isNaN(+name.slice(1))) {
            this[name] = undefined$1;
          }
        }
      }
    },

    stop: function() {
      this.done = true;

      var rootEntry = this.tryEntries[0];
      var rootRecord = rootEntry.completion;
      if (rootRecord.type === "throw") {
        throw rootRecord.arg;
      }

      return this.rval;
    },

    dispatchException: function(exception) {
      if (this.done) {
        throw exception;
      }

      var context = this;
      function handle(loc, caught) {
        record.type = "throw";
        record.arg = exception;
        context.next = loc;

        if (caught) {
          // If the dispatched exception was caught by a catch block,
          // then let that catch block handle the exception normally.
          context.method = "next";
          context.arg = undefined$1;
        }

        return !! caught;
      }

      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i];
        var record = entry.completion;

        if (entry.tryLoc === "root") {
          // Exception thrown outside of any try block that could handle
          // it, so set the completion value of the entire function to
          // throw the exception.
          return handle("end");
        }

        if (entry.tryLoc <= this.prev) {
          var hasCatch = hasOwn.call(entry, "catchLoc");
          var hasFinally = hasOwn.call(entry, "finallyLoc");

          if (hasCatch && hasFinally) {
            if (this.prev < entry.catchLoc) {
              return handle(entry.catchLoc, true);
            } else if (this.prev < entry.finallyLoc) {
              return handle(entry.finallyLoc);
            }

          } else if (hasCatch) {
            if (this.prev < entry.catchLoc) {
              return handle(entry.catchLoc, true);
            }

          } else if (hasFinally) {
            if (this.prev < entry.finallyLoc) {
              return handle(entry.finallyLoc);
            }

          } else {
            throw new Error("try statement without catch or finally");
          }
        }
      }
    },

    abrupt: function(type, arg) {
      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i];
        if (entry.tryLoc <= this.prev &&
            hasOwn.call(entry, "finallyLoc") &&
            this.prev < entry.finallyLoc) {
          var finallyEntry = entry;
          break;
        }
      }

      if (finallyEntry &&
          (type === "break" ||
           type === "continue") &&
          finallyEntry.tryLoc <= arg &&
          arg <= finallyEntry.finallyLoc) {
        // Ignore the finally entry if control is not jumping to a
        // location outside the try/catch block.
        finallyEntry = null;
      }

      var record = finallyEntry ? finallyEntry.completion : {};
      record.type = type;
      record.arg = arg;

      if (finallyEntry) {
        this.method = "next";
        this.next = finallyEntry.finallyLoc;
        return ContinueSentinel;
      }

      return this.complete(record);
    },

    complete: function(record, afterLoc) {
      if (record.type === "throw") {
        throw record.arg;
      }

      if (record.type === "break" ||
          record.type === "continue") {
        this.next = record.arg;
      } else if (record.type === "return") {
        this.rval = this.arg = record.arg;
        this.method = "return";
        this.next = "end";
      } else if (record.type === "normal" && afterLoc) {
        this.next = afterLoc;
      }

      return ContinueSentinel;
    },

    finish: function(finallyLoc) {
      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i];
        if (entry.finallyLoc === finallyLoc) {
          this.complete(entry.completion, entry.afterLoc);
          resetTryEntry(entry);
          return ContinueSentinel;
        }
      }
    },

    "catch": function(tryLoc) {
      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i];
        if (entry.tryLoc === tryLoc) {
          var record = entry.completion;
          if (record.type === "throw") {
            var thrown = record.arg;
            resetTryEntry(entry);
          }
          return thrown;
        }
      }

      // The context.catch method must only be called with a location
      // argument that corresponds to a known catch block.
      throw new Error("illegal catch attempt");
    },

    delegateYield: function(iterable, resultName, nextLoc) {
      this.delegate = {
        iterator: values(iterable),
        resultName: resultName,
        nextLoc: nextLoc
      };

      if (this.method === "next") {
        // Deliberately forget the last sent value so that we don't
        // accidentally pass it on to the delegate.
        this.arg = undefined$1;
      }

      return ContinueSentinel;
    }
  };

  // Regardless of whether this script is executing as a CommonJS module
  // or not, return the runtime object so that we can declare the variable
  // regeneratorRuntime in the outer scope, which allows this module to be
  // injected easily by `bin/regenerator --include-runtime script.js`.
  return exports;

}(
  // If this script is executing as a CommonJS module, use module.exports
  // as the regeneratorRuntime namespace. Otherwise create a new empty
  // object. Either way, the resulting object will be used to initialize
  // the regeneratorRuntime variable at the top of this file.
   module.exports 
));

try {
  regeneratorRuntime = runtime;
} catch (accidentalStrictMode) {
  // This module should not be running in strict mode, so the above
  // assignment should always work unless something is misconfigured. Just
  // in case runtime.js accidentally runs in strict mode, we can escape
  // strict mode using a global Function call. This could conceivably fail
  // if a Content Security Policy forbids using Function, but in that case
  // the proper solution is to fix the accidental strict mode problem. If
  // you've misconfigured your bundler to force strict mode and applied a
  // CSP to forbid Function, and you're not willing to fix either of those
  // problems, please detail your unique predicament in a GitHub issue.
  Function("r", "regeneratorRuntime = r")(runtime);
}
});

var getScanOptions = function getScanOptions(root) {
  var options = {};
  var name = p.join(root, ScanConfigFilename);

  if (fs.existsSync(name)) {
    Object.assign(options, JSON.parse(fs.readFileSync(name).toString()));
  }

  return options;
};
var defaultScanConvertor = function defaultScanConvertor(data) {
  return Array.from(data.entries()).reduce(function (list, _ref) {
    var key = _ref[0],
        item = _ref[1];
    return list.concat({
      id: key,
      defaultMessage: item.defaultMessage,
      description: item.description
    });
  }, []);
};
var getTranslatedDir = function getTranslatedDir(root) {
  var filename = p.join(root, 'l10n.config.js');
  var content = fs.existsSync(filename) ? require(filename) : null;
  var defaultDir = 'locales';
  var ret;

  if (!content) {
    ret = defaultDir;
    return p.join(root, ret);
  }

  try {
    var obj = content.middlewares;
    var match = obj.emit.match(/\?dest=([^?]+)/);
    ret = match ? match[1] : defaultDir;
  } catch (_) {
    ret = defaultDir;
  }

  return p.join(root, ret);
};

var globASync = /*#__PURE__*/util.promisify(glob);

var runPlguin = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(file, plugin) {
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return core.transformFileAsync(file, {
              presets: ['@babel/preset-env', '@babel/preset-react', '@babel/preset-typescript'],
              plugins: [['@babel/plugin-proposal-decorators', {
                legacy: true
              }], ['@babel/plugin-proposal-class-properties'], '@babel/plugin-syntax-dynamic-import', plugin]
            });

          case 2:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function runPlguin(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

var pattern = /[a-z]{2}(?:-[A-Z]{2})?/;
var keysMap = /*#__PURE__*/new Map( /*#__PURE__*/['id', 'defaultMessage', 'filename', 'loc', 'translatedMessage'].map(function (item) {
  return [item, item];
}));
/**
 * 从列表中查找最合适的 descriptor
 * 含有翻译信息最多，如果一样但是内容不一样，给出警告
 * @param list
 */

var findDescriptor = function findDescriptor(list) {
  var locales = list.map(function (item) {
    return Object.keys(item).filter(function (key) {
      return !keysMap.has(key) && pattern.test(key);
    });
  });
  var max = Math.max.apply(Math, locales.map(function (item) {
    return item.length;
  }));
  var items = [];
  locales.forEach(function (item, index) {
    if (item.length === max) {
      items.push(list[index]);
    }
  });

  if (max === 0 || items.length === 1) {
    var _items$find;

    var target = (_items$find = items.find(function (item) {
      return item.translatedMessage;
    })) != null ? _items$find : items[0];
    return target;
  }

  console.warn('存在相同翻译元素但翻译内容不一样:');
  throw Error(JSON.stringify(items, null, 2));
};
/**
 * 提取翻译数据
 *
 * @param root 艮目录
 * @param source 待提取的文件
 * @param dist 输出文件名
 */


var scan = /*#__PURE__*/function () {
  var _ref2 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee2(root, source, dist) {
    var options, internalTranslatedDataFilename, data, runScan, files, result, translatedData, _iterator, _step, _source, _iterator2, _step2, item, descriptor;

    return runtime_1.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            if (source === void 0) {
              source = 'src';
            }

            if (dist === void 0) {
              dist = I18nMessageFilename;
            }

            options = _extends({
              source: p.isAbsolute(source) ? source : p.join(root, source),
              dist: p.isAbsolute(dist) ? dist : p.join(root, dist),
              convertor: defaultScanConvertor
            }, getScanOptions(root));
            internalTranslatedDataFilename = p.join(p.dirname(options.dist), InternalTranslatedDataFilename);
            data = new Map();

            runScan = function runScan() {
              return run(false, options, data);
            };

            mkdirp.sync(p.dirname(options.dist));
            _context2.next = 9;
            return globASync(options.source + "/**/!(*.d).@(js|jsx|ts|tsx)");

          case 9:
            files = _context2.sent;
            _context2.next = 12;
            return Promise.all(files.map(function (file) {
              return runPlguin(file, runScan);
            }));

          case 12:
            result = new Map();
            translatedData = {};

            for (_iterator = _createForOfIteratorHelperLoose(data.values()); !(_step = _iterator()).done;) {
              _source = _step.value;

              for (_iterator2 = _createForOfIteratorHelperLoose(_source.values()); !(_step2 = _iterator2()).done;) {
                item = _step2.value;
                descriptor = findDescriptor(item);
                result.set(descriptor.id, descriptor);

                if (descriptor.translatedMessage) {
                  translatedData[descriptor.id] = descriptor.translatedMessage;
                }
              }
            }

            if (Object.keys(translatedData).length) {
              fs.writeFileSync(internalTranslatedDataFilename, "module.exports = " + JSON.stringify(translatedData) + ";");
            }

            fs.writeFileSync(options.dist, JSON.stringify(options.convertor(result)));

          case 17:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function scan(_x3, _x4, _x5) {
    return _ref2.apply(this, arguments);
  };
}();

var getExcelHeaders = function getExcelHeaders(locales) {
  return [{
    header: 'id',
    key: 'id'
  }].concat(Array.from(locales.values()).sort(function (a, b) {
    return b.localeCompare(a);
  }).map(function (item) {
    return {
      key: item,
      header: item,
      width: 50
    };
  }));
};

var mergePreviousExcelData = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(map, previousFilename) {
    var previousData, _iterator, _step, item, list;

    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            if (!fs.existsSync(previousFilename)) {
              _context.next = 5;
              break;
            }

            _context.next = 3;
            return readExcelData(previousFilename);

          case 3:
            previousData = _context.sent;

            for (_iterator = _createForOfIteratorHelperLoose(previousData.values()); !(_step = _iterator()).done;) {
              item = _step.value;

              if (map.has(item.id)) {
                map.set(item.id, _extends({}, item, map.get(item.id)));
              }
            }

          case 5:
            list = Array.from(map.values());
            return _context.abrupt("return", list.sort(function (a, b) {
              return JSON.stringify(a).length - JSON.stringify(b).length;
            }));

          case 7:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function mergePreviousExcelData(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();
var mergeInternalTranslatedData = function mergeInternalTranslatedData(map, translatedDataFilename) {
  if (fs.existsSync(translatedDataFilename)) {
    var data = require(translatedDataFilename);

    Object.entries(data).forEach(function (_ref2) {
      var id = _ref2[0],
          item = _ref2[1];

      if (map.has(id)) {
        map.set(id, _extends({}, map.get(id), item));
      }
    });
  }

  return map;
};
var writeDataToExcel = /*#__PURE__*/function () {
  var _ref3 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee2(locales, data, filename) {
    var book, sheet;
    return runtime_1.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            book = new Excel.Workbook();
            sheet = book.addWorksheet('data');
            sheet.columns = getExcelHeaders(locales);
            sheet.addRows(data);
            _context2.next = 6;
            return book.xlsx.writeFile(filename);

          case 6:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function writeDataToExcel(_x3, _x4, _x5) {
    return _ref3.apply(this, arguments);
  };
}();
var readExcelData = /*#__PURE__*/function () {
  var _ref4 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee3(filename) {
    var book, map, sheet, headers;
    return runtime_1.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            book = new Excel.Workbook();
            _context3.next = 3;
            return book.xlsx.readFile(filename);

          case 3:
            // @ts-ignore
            map = new Map();
            sheet = book.worksheets[0];

            if (sheet !== null && sheet.rowCount > 0) {
              headers = sheet.getRow(1).values.slice(1);
              sheet.eachRow(function (row, index) {
                if (index === 1) {
                  return;
                }

                var values = row.values;
                var item = values.slice(1);
                map.set(item[0], _extends({
                  id: item[0]
                }, item.slice(1).reduce(function (map, val, index) {
                  var _extends2;

                  return _extends({}, map, (_extends2 = {}, _extends2[headers[index + 1]] = val, _extends2));
                }, {})));
              });
            }

            return _context3.abrupt("return", map);

          case 7:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3);
  }));

  return function readExcelData(_x6) {
    return _ref4.apply(this, arguments);
  };
}();

var globASync$1 = /*#__PURE__*/util.promisify(glob);

var readData = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(file) {
    var content, locale, list;
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return fs.promises.readFile(file);

          case 2:
            _context.next = 4;
            return _context.sent.toString();

          case 4:
            content = _context.sent;
            locale = p.basename(file, p.extname(file));
            list = Object.entries(JSON.parse(content)).reduce(function (ret, _ref2) {
              var id = _ref2[0],
                  defaultMessage = _ref2[1];
              return ret.concat({
                id: id,
                defaultMessage: defaultMessage
              });
            }, []);
            return _context.abrupt("return", {
              locale: locale,
              list: list
            });

          case 8:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function readData(_x) {
    return _ref.apply(this, arguments);
  };
}();
/**
 * 将翻译后的数据 提取到 excel,如果在目录下存在 excel , 则合并
 *
 * @param root 根目录
 * @param source 目标目录
 * @param dist 输出 Excel 文件名
 */


var pickL18nMessages = /*#__PURE__*/function () {
  var _ref3 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee2(root, source, dist) {
    var sourceDir, files, distFilename, internalTranslatedDataFilename, data, map, locales, list;
    return runtime_1.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            if (source === void 0) {
              source = /*#__PURE__*/p.dirname(I18nMessageFilename);
            }

            if (dist === void 0) {
              dist = ExcelFilename;
            }

            sourceDir = getTranslatedDir(root);
            _context2.next = 5;
            return globASync$1(sourceDir + "/*.json");

          case 5:
            files = _context2.sent;
            distFilename = p.isAbsolute(dist) ? dist : p.join(sourceDir, dist);
            internalTranslatedDataFilename = p.join(p.isAbsolute(source) ? source : p.join(root, source), InternalTranslatedDataFilename);
            _context2.next = 10;
            return Promise.all(files.map(function (file) {
              return readData(file);
            }));

          case 10:
            data = _context2.sent;
            map = new Map();
            locales = new Set();
            data.forEach(function (_ref4) {
              var locale = _ref4.locale,
                  list = _ref4.list;
              var realLocale = locale in LocaleMap ? LocaleMap[locale] : locale;
              locales.add(realLocale);
              list.forEach(function (item) {
                var _extends2;

                if (!map.has(item.id)) {
                  map.set(item.id, {
                    id: item.id
                  });
                }

                var target = map.get(item.id);
                map.set(item.id, _extends({}, target, (_extends2 = {}, _extends2[realLocale] = item.defaultMessage, _extends2)));
              });
            });
            _context2.next = 16;
            return mergePreviousExcelData(mergeInternalTranslatedData(map, internalTranslatedDataFilename), distFilename);

          case 16:
            list = _context2.sent;
            _context2.next = 19;
            return writeDataToExcel(locales, list, distFilename);

          case 19:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function pickL18nMessages(_x2, _x3, _x4) {
    return _ref3.apply(this, arguments);
  };
}();

var insertImport = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(filename, locale, name) {
    var content;
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return fs.promises.readFile(filename);

          case 2:
            _context.next = 4;
            return _context.sent.toString();

          case 4:
            content = _context.sent;

            if (content && !content.includes(name)) {
              content = content.replace(/\s*export\s+default/, function (match) {
                return content.includes(name) ? match : "\nimport " + name + " from \"./" + locale + "/" + name + "\";" + match;
              });
              content = content.replace(/(,)?(\s*};?\s*$)/, function (_, $1, $2) {
                return ($1 || ',') + "\n  ..." + name + "," + $2;
              });
            } else if (!content) {
              content = "import " + name + " from \"./" + locale + "/" + name + "\";\n\nexport default {\n  ... " + name + "\n};\n    ";
            }

            _context.next = 8;
            return fs.promises.writeFile(filename, content);

          case 8:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function insertImport(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
}();
/**
 * 将 Excel 文件里的翻译数据分发到代码里
 *
 * @param root 根目录
 * @param dist 目标路径 , 参照 umi locales
 * @param excel Excel 文件
 */


var distribute = /*#__PURE__*/function () {
  var _ref2 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee3(root, dist, excel) {
    var sourceDir, distDir, excelFilename, data, list, keys, locales, dirs, collections;
    return runtime_1.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            if (dist === void 0) {
              dist = 'src/locales';
            }

            if (excel === void 0) {
              excel = ExcelFilename;
            }

            sourceDir = getTranslatedDir(root);
            distDir = p.isAbsolute(dist) ? dist : p.join(root, dist);
            excelFilename = p.join(sourceDir, excel);
            _context3.next = 7;
            return readExcelData(excelFilename);

          case 7:
            data = _context3.sent;

            if (!(data.size === 0)) {
              _context3.next = 10;
              break;
            }

            return _context3.abrupt("return");

          case 10:
            list = Array.from(data.values());
            keys = list.map(Object.keys).sort(function (a, b) {
              return b.length - a.length;
            });
            locales = keys[0].filter(function (item) {
              return item !== 'id';
            });
            dirs = locales.map(function (locale) {
              return p.join(distDir, locale);
            });
            collections = locales.map(function (locale) {
              return {
                locale: locale,
                data: list.reduce(function (map, item) {
                  var _extends2;

                  return _extends({}, map, (_extends2 = {}, _extends2[item.id] = item[locale], _extends2));
                }, {})
              };
            });
            dirs.map(mkdirp.sync);
            dirs.forEach(function (dir) {
              mkdirp.sync(dir);

              if (!fs.existsSync(dir + ".ts")) {
                fs.writeFileSync(dir + ".ts", '');
              }
            });
            _context3.next = 19;
            return Promise.all(collections.map( /*#__PURE__*/function () {
              var _ref4 = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee2(_ref3) {
                var locale, data, content, targetFilename;
                return runtime_1.wrap(function _callee2$(_context2) {
                  while (1) {
                    switch (_context2.prev = _context2.next) {
                      case 0:
                        locale = _ref3.locale, data = _ref3.data;
                        content = "export default " + JSON.stringify(data, null, 2) + ";";
                        targetFilename = p.join(distDir, locale, DistributionName + ".ts");
                        _context2.next = 5;
                        return Promise.all([fs.promises.writeFile(targetFilename, content), insertImport(p.join(distDir, locale + ".ts"), locale, DistributionName)]);

                      case 5:
                      case "end":
                        return _context2.stop();
                    }
                  }
                }, _callee2);
              }));

              return function (_x7) {
                return _ref4.apply(this, arguments);
              };
            }()));

          case 19:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3);
  }));

  return function distribute(_x4, _x5, _x6) {
    return _ref2.apply(this, arguments);
  };
}();

/**
 * 将自动提取到的数据 提取到 excel,如果在目录下存在 excel , 则合并
 *
 * @param root  根目录
 * @param source  目标路径
 * @param locale 默认语言
 */

var transform = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(root, source, excelFilename, locale) {
    var l18nSourceDir, content, internalTranslatedDataFilename, distFilename, map, locales, list;
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            if (source === void 0) {
              source = I18nMessageFilename;
            }

            if (excelFilename === void 0) {
              excelFilename = ExcelFilename;
            }

            if (locale === void 0) {
              locale = 'zh-CN';
            }

            source = p.isAbsolute(source) ? source : p.join(root, source);
            l18nSourceDir = getTranslatedDir(root);
            content = JSON.parse(fs.readFileSync(source).toString());
            internalTranslatedDataFilename = p.join(p.dirname(source), InternalTranslatedDataFilename);
            distFilename = p.isAbsolute(excelFilename) ? excelFilename : p.join(l18nSourceDir, ExcelFilename);
            map = new Map();
            locales = new Set(Object.values(LocaleMap));

            if (!p.isAbsolute(excelFilename)) {
              mkdirp.sync(l18nSourceDir);
            }

            content.forEach(function (item) {
              var _map$set;

              map.set(item.id, (_map$set = {
                id: item.id
              }, _map$set[locale] = item.defaultMessage, _map$set));
            });
            mergeInternalTranslatedData(map, internalTranslatedDataFilename);
            _context.next = 15;
            return mergePreviousExcelData(map, distFilename);

          case 15:
            list = _context.sent;
            _context.next = 18;
            return writeDataToExcel(locales, list, distFilename);

          case 18:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function transform(_x, _x2, _x3, _x4) {
    return _ref.apply(this, arguments);
  };
}();

var execAsync = /*#__PURE__*/util.promisify(child_process.exec);
/**
 * 使用 atool-ll10n 翻译
 */

var translate = /*#__PURE__*/(function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(root) {
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return execAsync('npx atool-l10n', {
              cwd: root
            });

          case 2:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function (_x) {
    return _ref.apply(this, arguments);
  };
})();

exports.shorthash = sh__default;
exports.BabelPluginUmiMesIntl = plugin;
exports.default = umiPlugin;
exports.distribute = distribute;
exports.id = genId;
exports.pickL18nMessages = pickL18nMessages;
exports.scan = scan;
exports.transform = transform;
exports.translate = translate;
//# sourceMappingURL=mes-babel-plugin-intl.cjs.development.js.map
